#version 330 core

// Interpolated values from the vertex shaders
//in vec3 fragmentColor;

in vec2 fragmentUV;
in vec3 normal_cs;
in vec3 lightdirection_cs[5];
in vec3 position_ws;
in vec3 eyedirection_cs;
in vec4 shadowCoord;

out vec4 fragColor;

uniform sampler2D colorTexture;
uniform sampler2DShadow shadowMap;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform vec3 lightposition_ws[5];

void main(void){
//	const vec3 lightposition_ws[5] = vec3[5](vec3(0.0, 9.0, -8.0), vec3(0.0, 9.0, -32.0), vec3(0.0, 9.0, -56.0), vec3(0.0, 9.0, -80.0), vec3(0.0, 9.0, -104.0));

	vec4 textureColor = texture(colorTexture, fragmentUV);
	vec4 materialColor = textureColor; //vec4(textureColor, 1.0);
	vec4 ambient = vec4(0.1, 0.1, 0.1, 1) * materialColor;
	vec4 specular = vec4(0.3, 0.3, 0.3, 1);
	vec4 lightColor = vec4(1.0, 1.0, 1.0, 0.0);
	float lightPower = 50.0f;
	//mat4 mv = viewMatrix * modelMatrix;

	fragColor = ambient;

	for (int u = 0; u < lightposition_ws.length(); ++u) {
		// Distance to the light
		float distance = length(lightposition_ws[u] - position_ws);

		// lightning
		vec3 n = normalize(normal_cs);
		vec3 l = normalize(lightdirection_cs[u]);
		float cosTheta = clamp(dot( n,l ), 0, 1);
		
		// speculars
		vec3 e = normalize(eyedirection_cs);
		vec3 r = reflect(-l,n);
		float cosAlpha = clamp(dot(e, r), 0, 1);

		// adapt to current light & shadowcoords etc
		float visibility = 1;
		float bias = clamp(0.005*tan(acos(cosTheta)), 0, 0.01);
		if (u == 1) {
			vec3 shadowMapCoord = shadowCoord.xyz/shadowCoord.w;
			float depth = texture(shadowMap, shadowMapCoord.xyz);
			visibility = (depth < (shadowMapCoord.z-bias)/shadowCoord.w) ? 0.0 : 1.0;
			//visibility = texture(shadowMap, vec3(shadowCoord.xy, (shadowCoord.z)/shadowCoord.w));
		} 

		fragColor += visibility * materialColor * lightColor * lightPower * cosTheta / (distance * distance) + // used to be (distance * distance) but I didn't like it for testing purposes
					visibility * specular * lightColor * lightPower * pow(cosAlpha, 10) / (distance * distance); // used to be (distance * distance) but I didn't like it for testing purposes
	}			
}