#version 330 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec2 uv;
layout(location = 2) in vec3 normal;

out vec3 normal_cs;
out vec2 fragmentUV;
out vec3 position_ws;
out vec3 eyedirection_cs;
out vec3 lightdirection_cs[5];
out vec4 shadowCoord;

// values from the main.cpp and/or the scene objects
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat4 modelMatrix;
uniform vec3 lightposition_ws[5];
uniform mat4 depthBiasVP;

void main() {
//	const vec3 lightposition_ws[5] = vec3[5](vec3(0.0, 9.0, -8.0), vec3(0.0, 9.0, -32.0), vec3(0.0, 9.0, -56.0), vec3(0.0, 9.0, -80.0), vec3(0.0, 9.0, -104.0));
	
	shadowCoord = depthBiasVP * modelMatrix * vec4(position, 1);
	
	fragmentUV = uv;
	
	gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(position, 1);

	// Position of the vertex, in worldspace : M * position
	position_ws = (modelMatrix * vec4(position,1)).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 vertexposition_cs = (viewMatrix * modelMatrix * vec4(position,1)).xyz;
	eyedirection_cs = vec3(0,0,0) - vertexposition_cs;


	for (int u = 0; u < lightposition_ws.length(); ++u) {
		// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.		
		vec3 lightposition_cs = ( viewMatrix * vec4(lightposition_ws[u], 1)).xyz;
		lightdirection_cs[u] = lightposition_cs + eyedirection_cs;
	}
	// Normal of the the vertex, in camera space
	normal_cs = ( viewMatrix * modelMatrix * vec4(normal, 0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
}